# ###########################################################################
# This file is part of the Alida-C++ library, an 
#
# Advanced Library for Integrated Development of Data Analysis Applications.
#
# Copyright (C) 2012 - @YEAR@
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Fore more information on Alida, visit
#
#    http://www.informatik.uni-halle.de/alida/
#
# ###########################################################################

# ### 
# Most recent change(s):
# 
# $Rev: 4830 $
# $Date: 2012-01-13 13:40:10 +0100 (Fr, 13 Jan 2012) $
# $Author: moeller $
# 
# ###

# #####################################################
export CXX		:= g++

############################################################################
# FLAG-SETTINGS
############################################################################

CXX_FLAGS += -fPIC -Wall -I$(ALIDA_CPP)/src
LD_FLAGS += -Wl,-rpath,$(ALIDA_CPP)/lib_$(ARCH)

LOCAL_CXX_FLAGS	= $(LOCAL_INCLUDES)
LOCAL_LD_FLAGS	= $(LOCAL_LIBS)


COMP_FLAGS	= 	$(CXX_FLAGS)		\
		  		$(LOCAL_CXX_FLAGS)	\
		  		$(LOCAL_DEFS)		\
		  		$(DEBUG_FLAGS) 

LINK_FLAGS	= $(LD_FLAGS) -L$(ALIDA_CPP)/lib_$(ARCH) $(LOCAL_LD_FLAGS)

DYNLIB_FLAGS    = -shared -Wl,-soname,

#############################################################################
# globally valid variable names
#############################################################################

LIBRARY			= $(LIBNAME).$(LIBSUFFIX)
DYNLIB			= $(LIBRARY).$(DYNLIB_VER_MAJOR).$(DYNLIB_VER_MINOR)
DYNLIB_M		= $(LIBRARY).$(DYNLIB_VER_MAJOR)

OBJS=	$(SRC:.cc=.o)
ESRC=	$(SRC:.cc=.E)

TARGET_OBJS	= $(TARGET_SRC:.cc=.o)

#############################################################################
# globally valid rules for all make calls
#############################################################################

$(TARGET):	$(OBJS) $(TARGET_OBJS)
	$(CXX) $^ $(LINK_FLAGS) $(TARGET_LIBS) -o $(TARGET) 

%.o:$(SOURCEDIRS)/%.cc $(SOURCEDIRS)/%.h
	$(CXX) -c $< $(COMP_FLAGS) -o $@

%.o:$(SOURCEDIRS)/%.cc
	$(CXX) -c $< $(COMP_FLAGS) -o $@

%.E:$(SOURCEDIRS)/%.cc $(SOURCEDIRS)/%.h
	$(CXX) -E $< $(COMP_FLAGS) -o $@

#############################################################################
# globally valid targets for make calls
#############################################################################

expand: $(ESRC)

clean:	
	-rm -f $(LIBRARY) $(DYNLIB) $(DYNLIB_M)
	-rm -f $(OBJS) $(ESRC)
	-rm -f $(TARGET_OBJS) $(TARGET)
	-rm -f $(TARGET_SRC.$(ARCH):.cc=.d) $(SRC.$(ARCH):.cc=.d)

dynlib: $(DYNLIB)

$(DYNLIB):	$(OBJS) 
	$(CXX) $(DYNLIB_FLAGS)$(DYNLIB_M) -o $(DYNLIB) $^
	@-mkdir $(LOCAL_LIB_DIR)
	@-ln -s $(DYNLIB) $(LIBRARY)
	@-ln -s $(DYNLIB) $(DYNLIB_M)
	@-ln -s $(SOURCEDIRS)/o.$(ARCH)/$(DYNLIB) $(LOCAL_LIB_DIR)/$(DYNLIB)
	@-ln -s $(DYNLIB) $(LOCAL_LIB_DIR)/$(DYNLIB_M)
	@-ln -s $(DYNLIB_M) $(LOCAL_LIB_DIR)/$(LIBRARY)

lib:	$(OBJS.$(ARCH)) 
	ar ruv $(LIBRARY) $^
	ranlib $(LIBRARY)
	@-ln -sf $(SOURCE)/$(BUILD)/$(LIBRARY) ../../../../../lib/

SUBDIR= $(subst /misc/vol_sol/agsoft/src/topas/src/libs/,,$(PWD))
install: 
	@-echo "Installing header and libs..."
	@-mkdir -p $(INSTALL-PATH)/include/topas/$(SUBDIR)
	@-cp ../*.h $(INSTALL-PATH)/include/topas/$(SUBDIR)
	@-cp ./$(DYNLIB) $(LIB-PATH)/$(DYNLIB)
	@-ln -s $(LIB-PATH)/$(DYNLIB) $(LIB-PATH)/$(DYNLIB_M)
	@-ln -s $(LIB-PATH)/$(DYNLIB) $(LIB-PATH)/$(basename $(DYNLIB_M))

localinstall: 
	@ echo "Nothing to install local operator directory..." 

test:	$(TARGET)

#############################################################################
# dependencies
#############################################################################

%.d: ../%.cc
	$(CXX) -MM $< $(COMP_FLAGS) $< > $@.$$$$; 					\
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g' < $@.$$$$ > $@; \
	rm -f $@.$$$$

-include $(SRC:.cc=.d) $(TARGET_SRC:.cc=.d)
#############################################################################
