/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#include <math.h>
#include <dlfcn.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include "dataio/ALDDataIOManager.h"

using namespace std;

using namespace Alida;

/**
 * By calling this function (addressed by the generic macro #REGISTER_PROVIDER)
 * data I/O providers can register themselves for specific data types. The
 * function fills the registration map of an I/O manager, storing the data
 * type identifier string and an associated prototype provider instance.
 * <p>
 * To ease the registration process and allow for dynamic registration during
 * program initialization and prior to the call of the main()-function of a
 * program dynamic initialization of static variables in C++ is used here.
 * Therefore the method returns a pointer to the prototype instance which can
 * be assigned to a global variable ensuring dynamic registration of the
 * provider. The assignment to static variables is transiently handled by the
 * #REGISTER_PROVIDER macro, hence, no additional work remains for the
 * programmer.
 *
 * @param type	Identifier string for the registered data type.
 * @param p			Prototype provider class instance.
 * @return Pointer to provided prototype instance.
 */
ALDDataIO* Alida::ALDDataIOManager::registerProvider(string type, ALDDataIO *p)
{
	if (this->registrationTable == NULL)
		this->registrationTable = new map<string, ALDDataIO*>();
	cout << "[IOManager] Registering provider for type " << type << endl;
	(*registrationTable)[type] = p;
	map<string, ALDDataIO*>::iterator it;
	return p;
}

/**
 * This function prints all registered providers to standard out.
 */
void Alida::ALDDataIOManager::printRegistrationTable()
{
	cout << "Registered providers:" << endl;
	map<string, ALDDataIO*>::iterator it;
	for (it = this->registrationTable->begin();
					it != this->registrationTable->end(); ++it)
		cout << "Index: " << it->first << endl;
}
