/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#ifndef IOM_H_INCLUDED
#define IOM_H_INCLUDED

#include <map>
#include <string>

#include "dataio/provider/ALDDataIO.h"

/** register a provider of class PROVIDER_CLASS to provide DataIO for
 * class TYPE. The provider extends the class PROVIDER_TYPE
 * which in turn is to be a sub class of Provider
 */
#define REGISTER_PROVIDER(PROVIDER_CLASS,TYPE,PROVIDER_TYPE) \
	ALDDataIO* PROVIDER_CLASS ## TYPE ## _myProvider = \
		ALDDataIOManager ## PROVIDER_TYPE::Instance().registerProvider(typeid( new TYPE()).name(), new PROVIDER_CLASS());

/**
 * @defgroup provider_registrations Registrations of providers
 */
/* @{ */
/* @} */

namespace Alida {

/// Base class for I/O managers in Alida-C++.
/**
 * This class yields the base class for all I/O managers. I/O managers are
 * supposed to act as some kind of gateway between<br> generic operator runners
 * for command line or GUI and data type specific I/O providers. <br> The
 * managers allow to keep all data type specific knowledge solely inside of
 * the corresponding provider classes without preventing generic data I/O.
 * <p>
 * To allow I/O managers to fulfill their work all data I/O providers are
 * requested to register themselves in the corresponding I/O manager class(es)
 * during initialization. <br> Each manager holds a hash of available provider
 * class, each associated with a class type identifier string.<br>
 * The hash is filled during the registration stage adopting the generic
 * macro #REGISTER_PROVIDER.
 * <p>
 * Upon I/O requests the manager can then clone appropriate provider instances
 * from its hash and delegate the I/O request accordingly.
 * <p>
 * Note that the Alida concepts recommends I/O managers to be implemented as
 * singletons. Thus, constructors and destructors of this class are protected.
 * <br>
 * Only instances of subclasses should be used for generic operator execution,
 * and these should likewise be addressed as singletons.<br>
 * For details on this issue and an example refer to the documentation
 * of the class ALDIOManagerCmdline.
 */
class ALDDataIOManager
{
protected:

	/// Default constructor, hidden to guarantee singleton property.
	ALDDataIOManager() {};

	/// Default destructor.
	virtual ~ALDDataIOManager() {};

	/// Hash of registered providers.
	std::map<std::string, ALDDataIO*>* registrationTable;

public:

	/// Convenience function to output the contents of the registration table.
	void printRegistrationTable();

	/// Function to register a provider class for a specific data type.
	virtual ALDDataIO* registerProvider(std::string type, ALDDataIO *provider);

};

}

#endif /* IOM_H_INCLUDED */
