/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 88 $
 * $Date: 2012-03-11 09:27:54 +0100 (So, 11 Mrz 2012) $
 * $Author: posch $
 *
 */

#include <dlfcn.h>
#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include "dataio/ALDDataIOManagerCmdline.h"
#include "dataio/provider/cmdline/ALDDataIOCmdline.h"

using namespace std;

using namespace Alida;

/**
 * This function handles data I/O input requests from command line operator
 * runners. <br> According to the given type identifier string the manager
 * calls a corresponding provider class instance and passes the input string
 * to this provider.<br>
 * The provider converts the given input string into an object of
 * the requested data type and hands it back to the manager.
 *
 * @param input		Input string to be processed.
 * @param	type		Identifier string for requested object type.
 * @return Generated object of requested type.
 */
void* Alida::_ALDDataIOManagerCmdline::readData(string input, string type)
{
	cout << "Reading " << type << "-data from string " << input << endl;
	ALDDataIOCmdline* provObjCmd;
	if (this->registrationTable == NULL)
		cout << "Table is null" << endl;
	if (this->registrationTable->find(type) == this->registrationTable->end()) {
		cerr << "Generate provider - no prototype found!" << endl;
		return NULL;
	}
	ALDDataIO* provObj = (*(this->registrationTable))[type];
	provObjCmd = dynamic_cast<ALDDataIOCmdline*>(provObj);
	if (provObjCmd == NULL)
		cout << "Object is null!" << endl;
	return provObjCmd->readData(input, type);
}

/**
 * This function transfers the given input data to an output string by
 * delegating the I/O request to an appropriate data I/I provider.
 * The provider class is selected according to the given type identifier
 * string.
 *
 * @param format	Formatting options for the output string.
 * @param	type		Identifier string for object type.
 * @param data		Data object to put out.
 * @return Generated string with output values.
 */
string
Alida::_ALDDataIOManagerCmdline::writeData(string input,string type,void *data)
{
	ALDDataIOCmdline* provObjCmd;
	if (this->registrationTable == NULL)
		cout << "Table is null" << endl;
	if (this->registrationTable->find(type) == this->registrationTable->end()) {
		cerr << "Generate provider - no prototype found!" << endl;
		return string();
	}
	ALDDataIO* provObj = (*(this->registrationTable))[type];
	provObjCmd = dynamic_cast<ALDDataIOCmdline*>(provObj);
	if (provObjCmd == NULL)
		cout << "Object is null!" << endl;
	return provObjCmd->writeData(input, type, data);
}
