/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 34 $
 * $Date: 2012-02-18 20:29:54 +0100 (Sa, 18 Feb 2012) $
 * $Author: moeller $
 *
 */

#ifndef IOM_CMDLINE_H_INCLUDED
#define IOM_CMDLINE_H_INCLUDED

#include <map>
#include <string>

#include "../external/loki-0.1.7/include/loki/Singleton.h"

#include "dataio/ALDDataIOManager.h"

/*
 * Definition for the Loki library to enable multithread support for the
 * singleton class.
 */
#define LOKI_OBJECT_LEVEL_THREADING

namespace Alida {

/// I/O manager class for managing command line data I/O.
/**
 * This hidden class coordinates the data I/O of parameter objects for
 * running operators from command line. <br>
 * As the class is not accessible from
 * outside, the corresponding singleton should be used to query the class
 * for providers.
 */
class _ALDDataIOManagerCmdline : public ALDDataIOManager
{
protected:

	/// Default constructor.
	_ALDDataIOManagerCmdline() {};

	/// Default destructor.
	virtual ~_ALDDataIOManagerCmdline() {};

	/*
	 * Friend declarations.
	 */
	friend class Loki::CreateUsingNew<Alida::_ALDDataIOManagerCmdline>;

public:

	/// Method to parse object data from a given string.
	void* readData(std::string, std::string);

	/// Method to transform given data to a suitable output format.
	std::string writeData(std::string format, std::string type, void* data);
};

}

/**
 * @defgroup singletons Singletons
 */
/* @{ */

/// Singleton type for accessing the command line data I/O manager class.
typedef Loki::SingletonHolder<Alida::_ALDDataIOManagerCmdline,
		Loki::CreateUsingNew,
		Loki::DefaultLifetime,
		Loki::SingleThreaded
> ALDDataIOManagerCmdline;

/* @} */

#endif /* IOM_CMDLINE_H_INCLUDED */
