/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#ifndef PROVIDER_H_INCLUDED
#define PROVIDER_H_INCLUDED

#include <map>
#include <string>
#include <typeinfo>

/** return an instance of this provider
 */
#define IMPLEMENT_CLONE(TYPE) \
	ALDDataIO* clone() const { return new TYPE(*this); }

namespace Alida {

/// Abstract class definition for data I/O providers in Alida.
/**
 * Data I/O providers are classes that allow for input and output of
 * specific data types via different modalities. <br> E.g., for commandline
 * interaction with operators corresponding providers are requested to
 * convert textual input to appropriate parameter objects, or to transform
 * a given data object to a suitable textual representation.
 * <p>
 * Data I/O providers are requested to register themselves on initialization
 * in the corresponding I/O manager classes, e.g. in ::ALDDataIOManagerCmdline.
 * <br>
 * To this end in the file of the common super class ALDDataIOManager of all
 * I/O managers the macro #REGISTER_PROVIDER is defined to ease this
 * registration procedure.
 */
class ALDDataIO
{
 protected:

	/// Default destructor.
	virtual ~ALDDataIO() {};

};

}

#endif /* PROVIDER_H_INCLUDED */
