/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include "dataio/ALDDataIOManagerCmdline.h"
#include "dataio/provider/cmdline/ALDNativeDatatypesDataIO.h"

using namespace std;

using namespace Alida;

/**
 * \ingroup provider_registrations
 * Register provider for native double data.
 */
REGISTER_PROVIDER(ALDNativeDatatypesDataIO,double,Cmdline);

/**
 * \ingroup provider_registrations
 * Register provider for native int data.
 */
REGISTER_PROVIDER(ALDNativeDatatypesDataIO,int,Cmdline);

/**
 * \ingroup provider_registrations
 * Register provider for native float data.
 */
REGISTER_PROVIDER(ALDNativeDatatypesDataIO,float,Cmdline);

/**
 * This function converts the given input string into an object of the
 * requested data type.
 *
 * @param input		Input string to be processed.
 * @param	type		Identifier string for requested object type.
 * @return Generated object of requested type.
 */
void*
ALDNativeDatatypesDataIO::readData(string input, string type) const
{
	if (typeid(new float()).name() == type) {
		cout << "Reading a float..." << endl;
		float* retFloat = new float();
		sscanf(input.c_str(),"%f",retFloat);
		return retFloat;
	}
	else if (typeid(new double()).name() == type) {
		cout << "Reading a double..." << endl;
		double *retDouble = new double();
		(*retDouble) = atof(input.c_str());
		return retDouble;
	}
	else if (typeid(new int()).name() == type) {
		cout << "Reading an int..." << endl;
		int *retInt = new int();
		(*retInt) = atoi(input.c_str());
		return retInt;

	}
	else {
		cerr << "ALDNativeDatatypesDataIO: unknown type requested! Error!" << endl;
		return NULL;
	}
}

/**
 * This function transfers the given input data to an output string.
 *
 * @param format	Formatting options for the output string.
 * @param	type		Identifier string for object type.
 * @param data		Data object to put out.
 * @return Generated string with output values.
 */
string ALDNativeDatatypesDataIO::writeData(
															string format, string type, void *data) const
{
	ostringstream outString;
	if (typeid(new float()).name() == type) {
		outString.setf(ios_base::scientific,ios_base::floatfield);
		outString.precision(10);
		outString << *((float*)data);
	}
	else if (typeid(new double()).name() == type) {
		outString.setf(ios_base::scientific,ios_base::floatfield);
		outString.precision(10);
		outString << *((double*)data);
	}
	else if (typeid(new int()).name() == type) {
		outString.precision(10);
		outString << *((int*)data);
	}
	else {
		outString << "ALDNativeDatatypesDataIO: unknown type in writing data!";
	}
	return outString.str();
}

