/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include "demo/operators/DemoOperator.h"
#include "operator/ALDOperatorManager.h"

using namespace std;

/**
 * \ingroup operator_registrations
 * Register demo operator.
 */
REGISTER_OPERATOR(DemoOperator);

/**
 * The constructor of an operator is supposed to take care of proper parameter
 * registration. Using the macro #DEFINE_PARAMETER defined in the file of
 * class ALDOperator each parameter that should be visible from outside needs
 * to be declared.
 * <p>
 * For this operator as a toy example four input parameters of native data
 * types are defined and one output parameter of type double.<br>
 * Note that each parameter definition is supposed to provide information about
 * - the parameter's data type
 * - its name
 * - the parameter direction
 * - whether it is required or not
 * - whether it is supplemental or not
 * - a label (e.g., to be used in GUIs)
 * - an explanation
 * - an optional default value
 */
DemoOperator::DemoOperator()
{
  DEFINE_PARAMETER( float, floatval, PARAMETER_IN,
  		true, false, "a float val", "This is a float value", NULL);

  DEFINE_PARAMETER( double, doubleval, PARAMETER_IN,
  		true, false, "a double val", "This is a double value", NULL);

  DEFINE_PARAMETER( int, intval, PARAMETER_IN,
  		true, false, "an int val", "This is an int value", NULL);

  DEFINE_PARAMETER( std::string, stringval, PARAMETER_IN,
  		true, false, "a string val", "This is a string value", NULL);

  DEFINE_PARAMETER( double, resultval, PARAMETER_OUT,
  		true, false, "a double val",
							 "This is the result, sum of floatval and doubleval", NULL);
}

/**
 * This function implements its abstract counterpart from the operator base
 * class and contains the functional core of this operator.<br>
 * In this specific example the function does nothing reasonable, but just
 * demonstrates how to access defined input parameters, and how to correctly
 * fill the operator's output parameters.
 */
void DemoOperator::operate()
{
  std::cout << "DemoOperator called..." << endl << endl;

  printf("Value of my parameter named floatval is %.8f\n",
    *((float*)(this->getParameter("floatval"))));

  printf("Value of my parameter named doubleval is %.20f\n",
    *((double*)(this->getParameter("doubleval"))));
  printf("Value of my parameter named intval is %d\n",
    *((int*)(this->getParameter("intval"))));
  std::cout << "Value of my parameter named stringval is " <<
    *((std::string*)(this->getParameter("stringval"))) << std::endl;

  double *result = new double();
  (*result) = (double)(*((float*)(this->getParameter("floatval")))) +
  		        *((double*)(this->getParameter("doubleval")));
  this->setParameter("resultval",result);
  std::cout << endl << "... operate() finished!" << std::endl;
}

/** Demonstrate custom validation, just require floatval to be non-negativ
 */ 
void DemoOperator::validateCustom() {
    if ( *((float*)(this->getParameter("floatval"))) < 0 ) {
        throw new ALDException( "DemoOperator", "validateCustom", "floatval is required to be non-negativ");
    }
}
