/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 111 $
 * $Date: 2012-03-12 16:00:37 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#ifdef OPENCV_SUPPORT

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include <opencv/cv.h>

#include "demo/operators/DemoOperatorOpenCV.h"
#include "operator/ALDOperatorManager.h"

/**
 * \ingroup operator_registrations
 * Register demo operator.
 */
REGISTER_OPERATOR(DemoOperatorOpenCV)

/**
 * The constructor is supposed to take care of proper parameter
 * registration. For this demo operator just two input parameters, i.e.
 * an image and a standard deviation are defined, and a single output
 * parameter containing the blurred version of the input image.
 */
DemoOperatorOpenCV::DemoOperatorOpenCV()
{
  DEFINE_PARAMETER(IplImage, inputImg, PARAMETER_IN,
  		true, false, "Input image", "Input image to be Gaussian filtered", NULL);

  DEFINE_PARAMETER(float, sigma, PARAMETER_IN,
  		true, false, "Sigma", "Stddev of Gaussian", new float(1.4));

  DEFINE_PARAMETER(IplImage, outputImg, PARAMETER_OUT,
  		true, false, "Output image", "Gaussian filtered image", NULL);
}

/**
 * This function implements its abstract counterpart from the operator base
 * class and contains the functional core of this operator.<br>
 * Here it applies the OpenCV function cvSmooth() to the given input image
 * adopting the specified standard deviation.<br>
 * In this specific example the function does nothing reasonable, but just
 * demonstrates how to access defined input parameters, and how to correctly
 * fill the operator's output parameters.
 */
void DemoOperatorOpenCV::operate()
{
	IplImage* image = ((IplImage*)(this->getParameter("inputImg")));
	std::cout << "Image size: " << image->width << std::endl;
	IplImage* outimage=
			cvCreateImage(cvSize(image->width,image->height),IPL_DEPTH_8U,image->nChannels);
	cvSmooth( image, outimage, CV_GAUSSIAN,
			*((float*)this->getParameter("sigma")),*((float*)this->getParameter("sigma")));
	this->setParameter("outputImg", outimage);
}

#endif /* OPENCV_SUPPORT */
