/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 101 $
 * $Date: 2012-03-12 09:47:11 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#ifdef OPENCV_SUPPORT

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include <opencv/cv.h>

#include "demo/operators/Thres.h"
#include "operator/ALDOperatorManager.h"

/**
 * \ingroup operator_registrations
 * Register the operator
 */
REGISTER_OPERATOR(Thres)

/**
 * The constructor is supposed to take care of proper parameter
 * registration. 
 * See openCV documentation
 */
Thres::Thres()
{
  DEFINE_PARAMETER(IplImage, inputImg, PARAMETER_IN,
  		true, false, "Input image", "Input image to be Gaussian filtered", NULL);

  DEFINE_PARAMETER(double, threshold, PARAMETER_IN,
  		true, false, "Threshold", "Threshold to apply", NULL);

  DEFINE_PARAMETER(double, fgvalue, PARAMETER_IN,
  		true, false, "Foreground Value", "Value of foregound pixels, i.e. pixels with intensitiy above threshold", NULL);

  DEFINE_PARAMETER(IplImage, outputImg, PARAMETER_OUT,
  		true, false, "Output image", "Gaussian filtered image", NULL);
}

/**
 * This operator applies the OpenCV function cvTreshhold() to the given input image.
 * For details see OpenCV documentation
 */
void Thres::operate()
{
	IplImage* image = ((IplImage*)(this->getParameter("inputImg")));
	std::cout << "Image size: " << image->width << std::endl;
	IplImage* outimage=
			cvCreateImage(cvSize(image->width,image->height),IPL_DEPTH_8U,image->nChannels);
	cvThreshold( image, outimage, 
			*((double*)this->getParameter("threshold")),
			*((double*)this->getParameter("fgvalue")),
			CV_THRESH_BINARY);
	this->setParameter("outputImg", outimage);
}

#endif /* OPENCV_SUPPORT */
