/*
 * This file is part of the Alida-C++ library, an
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 111 $
 * $Date: 2012-03-12 16:00:37 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#ifdef OPENCV_SUPPORT

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "dataio/ALDDataIOManagerCmdline.h"
#include "demo/providers/OpenCVImageDataIO.h"

using namespace std;

using namespace Alida;

/**
 * \ingroup provider_registrations
 * Register provider for OpenCV IplImage data type.
 */
REGISTER_PROVIDER(OpenCVImageDataIO,IplImage,Cmdline);

/**
 * This function interprets the given input string as filename and
 * tries to read an image from the specified location.
 *
 * @param input		Name of file to be read.
 * @param	type		Image type requested.
 * @return Generated openCV image.
 */
void* OpenCVImageDataIO::readData(string input, string type) const
{
	IplImage *img = cvLoadImage(input.c_str(),-1);
	return img;
}

/**
 * This function outputs/displays a given image. If the format string
 * just contains "-", the image is displayed on screen, otherwise the
 * format string is interpreted as filename where to save the image.
 * The returned string is empty in both cases.
 *
 * @param format	Output filename.
 * @param	type		Identifier string for object type.
 * @param data		Image to be displayed/saved.
 * @return Empty string.
 */
string
OpenCVImageDataIO::writeData(
					std::string format, std::string type, void *data) const
{
	string outString;
	if (data == NULL) {
		cerr << "Image is NULL, nothing to write..." << endl;
		return outString;
	}
	IplImage *img = (IplImage*)(data);

	if (format == "-") {
		// display the image on screen
		cvNamedWindow("Result Window",1);
		cvShowImage("Result Window",img);
		while(1) {
			if (cvWaitKey( 100) == 27)
				break;
		}
		cvDestroyWindow("Result Window");
	}
	else {
		// save to file
		cvSaveImage(format.c_str(),img);
	}
	return outString;
}

#endif /* OPENCV_SUPPORT */
