/*
 * This file is part of Alida-C++ library for
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 47 $
 * $Date: 2012/03/06 20:50:22 $
 * $Author: posch $
 *
 */

#ifndef OPPARAMETERDESCRIPTOR_H_INCLUDED
#define OPPARAMETERDESCRIPTOR_H_INCLUDED

/**
 * @file 	ALDOpParameterDescriptor.h
 * @brief	Contains the declaration of ALDOpParameterDescriptor.
 *
 * In addition, here the data type ALDParameterDirection is defined by
 * which the direction of an operator parameter can be specified.
 */

#include <string>
#include <typeinfo>

namespace Alida {

/// Enumerates the different directions an operator parameter may have.
typedef enum { 	PARAMETER_IN,
								PARAMETER_OUT,
								PARAMETER_INOUT} ALDParameterDirection;

/// Class to describe parameters of an operator.
class ALDOpParameterDescriptor
{
 public:

  /// Name of the parameter.
  const std::string name;
  /// Type identifier string.
  const std::string typeIdStr;
  /// User-readable type name.
  const std::string typeNiceName;
  /// Direction of parameter, i.e. IN, INOUT or OUT.
  const ALDParameterDirection direction;
  /// Flag for indicating if parameter is required or not.
  const bool required;
  /// Flag for indicating if parameter is supplemental or not.
  const bool supplemental;
  /// Expressive label describing the parameter.
  const std::string label;
  /// Short explanation of the parameters intention.
  const std::string explanation;
  /// Default value for the parameter.
  const void* defaultValue;
  /// Value of the parameter.
  void* value;

  /// Default constructor.
  ALDOpParameterDescriptor( std::string _name, std::string _typeIdStr,
    		std::string _typeNiceName, ALDParameterDirection _direction,
    		bool _required, bool _supplemental, std::string _label,
    		std::string _explanation, void* _defaultValue);

  /// Default destructor.
  ~ALDOpParameterDescriptor() {
  	//TODO: delete value and defaultValue
  }
};

}

#endif /* OPPARAMETERDESCRIPTOR_H_INCLUDED */
