/*
 * This file is part of Alida-C++ library for
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 102 $
 * $Date: 2012-03-12 10:30:51 +0100 (Mo, 12. Mär 2012) $
 * $Author: posch $
 *
 */

#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include "operator/ALDOperator.h"

using namespace std;

using namespace Alida;

/**
 * This functions is the only entry point for executing operators in Alida<br>
 * It basically calls an operator's operate() function.<br>
 * An Exception is thrown if either validation fails.
 * In addition, 
 * automatic process documentation will be done here in the future.
 */
void ALDOperator::runOp() throw (ALDException)
{
  cout << endl << "Entering runOp()..." << endl << endl;

  this->validate();
  this->operate();

  cout << endl << "...leaving runOp()!" << endl;
}

/*
 *  The generic part of this validation checks that each required parameter
 *  and input is set to a non-null value.
 *  The custom part of validation is operator depended and implemented
 * via the method <code>validateCustom</code>.
 *  An Exception is thrown if either validation fails.
*/
void ALDOperator::validate() throw (ALDException)
{
    map<string, ALDOpParameterDescriptor*> params = this->getParameters();
    map<string, ALDOpParameterDescriptor*>::const_iterator it;
    for (it = params.begin(); it != params.end(); ++it) {
        if ( (it->second->direction == PARAMETER_IN ||
              it->second->direction == PARAMETER_INOUT ) &&
             it->second->value == NULL) {
	    throw ALDException( "ALDOperator", "validate",  it->second->name + " is required, but NULL");
        } 
    } 

    validateCustom();
}

/**
 * @return Reference to the map of registered parameters.
 */
std::map<std::string, ALDOpParameterDescriptor*>& ALDOperator::getParameters()
{
  return this->parameters;
}

/**
 * @param	name	Name of the parameter the value is requested for.
 * @return Pointer to the corresponding parameter value.
 */
void* ALDOperator::getParameter(std::string name)
{
  return this->parameters[name]->value;
}

/**
 * @param	name		Name of the parameter to be set.
 * @param val			Value the parameter should retain.
 */
void ALDOperator::setParameter(std::string name, void* val)
{
  this->parameters[name]->value = val;
}
