/*
 * This file is part of Alida-C++ library for
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 96 $
 * $Date: 2012-03-11 17:23:38 +0100 (So, 11. Mär 2012) $
 * $Author: moeller $
 *
 */

#ifndef OPERATOR_H_INCLUDED
#define OPERATOR_H_INCLUDED

#include <map>
#include <string>
#include <typeinfo>

#include "operator/ALDOpParameterDescriptor.h"
#include "exceptions/ALDException.h"

namespace Alida {

/** Macro to define a parameter with NAME of given TYPE and to forth
 */
#define DEFINE_PARAMETER( TYPE, NAME, DIRECTION, REQUIRED, SUPPLEMENTAL, LABEL, EXPLANATION, DEFAULTVALUE ) \
	this->parameters[#NAME] = \
		new ALDOpParameterDescriptor( #NAME, typeid( new TYPE()).name(), #TYPE, \
				DIRECTION, REQUIRED, SUPPLEMENTAL, LABEL, EXPLANATION, DEFAULTVALUE);

/// Core class of the operator concept defining Alida operators.
class ALDOperator
{
 protected:

  /// Map containing list of registered parameters.
  std::map<std::string, ALDOpParameterDescriptor*> parameters;

	/// Default destructor.
  virtual ~ALDOperator() {};

  /// Method to implement operator functionality.
  virtual void operate() = 0;

 public:

  /// Method defining the entry point for operator execution.
  void runOp() throw (ALDException);

  /// Validates the parameters of this operator.
  void validate() throw (ALDException);

  /// Operator specific validation of parameters which should be overridden
  virtual void validateCustom() {};

  /// Function returning list of registered parameters.
  std::map<std::string, ALDOpParameterDescriptor*>& getParameters();

  /// Returns the value of the specified parameter.
  void* getParameter(std::string);

  /// Sets the specified parameter to the given value.
  void setParameter(std::string, void*);
};

}

#endif /* OPERATOR_H_INCLUDED */
