/*
 * This file is part of Alida-C++ library for
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 34 $
 * $Date: 2012/03/06 20:50:22 $
 * $Author: posch $
 *
 */

#include <dlfcn.h>
#include <math.h>
#include <stdio.h>

#include <fstream>
#include <iostream>
#include <string>

#include "operator/ALDOperatorManager.h"

using namespace std;

using namespace Alida;

/**
 * The constructor of the operator manager takes care of proper initialization
 * of its operator hash.
 */
Alida::_ALDOperatorManager::_ALDOperatorManager()
{
	this->operatorHash = new map<string, ALDOperator*>();
}

/**
 * Each operator that is supposed to be generically executed is required to
 * register itself in this class using this registration function. <br>
 * The function basically adds a prototype instance of the operator to the
 * manager's operator hash for later reference. In addition, it returns a
 * pointer to the prototype instance to facilitate dynamic registration
 * during a program's initialization phase (refer to
 * Alida::ALDDataIOManager::registerProvider for details on this mechanism).
 *
 * @param name		Operator name to be registered.
 * @param	op			Prototype instance of operator class.
 * @return Pointer to prototype instance.
 */
ALDOperator*
Alida::_ALDOperatorManager::registerOperator(string name, ALDOperator *op)
{
	cout << "[OperatorManager] Registering operator " << name << endl;
	(*operatorHash)[name] = op;
	return op;
}

/**
 * This function prints all registered operator name to standard out.
 */
void Alida::_ALDOperatorManager::printOperatorHash()
{
	cout << "Registered operators:" << endl;
	map<string, ALDOperator*>::iterator it;
	for (it = this->operatorHash->begin(); it != this->operatorHash->end(); ++it)
		cout << "Index: " << it->first << endl;
}

/**
 * This function returns a pointer to the prototype instance of the
 * requested operator.
 *
 * @param		name of desired operator.
 * @return Pointer to a prototype instance, might be NULL.
 */
ALDOperator* Alida::_ALDOperatorManager::getOperator(string name)
{
	return (*(this->operatorHash))[name];
}
