/*
 * This file is part of Alida-C++ library for
 *
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2012 - @YEAR@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/*
 * Most recent change(s):
 *
 * $Rev: 10 $
 * $Date: 2012/03/06 20:50:22 $
 * $Author: posch $
 *
 */

#ifndef OPERATOR_MANAGER_H_INCLUDED
#define OPERATOR_MANAGER_H_INCLUDED

#include <map>
#include <string>
#include <typeinfo>

#include "../external/loki-0.1.7/include/loki/Singleton.h"

#include "operator/ALDOperator.h"
#include "operator/ALDOpParameterDescriptor.h"

/** register a provider of class PROVIDER_CLASS to provide DataIO for
 * class TYPE. The provider extends the class PROVIDER_TYPE
 * which in turn is to be a sub class of Provider
 */
#define REGISTER_OPERATOR(OPERATOR_CLASS) \
	ALDOperator* OPERATOR_CLASS ## _myOperator = \
		ALDOperatorManager::Instance().registerOperator(#OPERATOR_CLASS, new OPERATOR_CLASS());

/**
 * @defgroup operator_registrations Registrations of operators
 */
/* @{ */
/* @} */

/*
 * Definition for the Loki library to enable multithread support for the
 * singleton class.
 */
#define LOKI_OBJECT_LEVEL_THREADING

namespace Alida {

/// Hidden class for operator instance management.
/**
 * In this class all operators register on program initialization
 * prior to the run of the main()-function. Afterwards this class can be
 * queried for instances of registered operators, e.g., for generic execution.
 *
 * The class is encapsulated into a singleton, i.e. not directly
 * accessible from outside. <br>Instead, refer to the data type
 * ::ALDOperatorManager for using oeprator manager functionality.
 */
class _ALDOperatorManager
{
protected:

	/// Map containing registered operators.
	std::map<std::string, ALDOperator*>* operatorHash;

	/// Default constructor.
	_ALDOperatorManager();

	/// Default destructor.
	virtual ~_ALDOperatorManager() {};

	/*
	 * Friend declarations.
	 */
	friend class Loki::CreateUsingNew<Alida::_ALDOperatorManager>;

public:
  
	/// Registration function for operators.
  ALDOperator* registerOperator(std::string name, ALDOperator *op);

  /// Returns operator instance corresponding to provided classname.
  ALDOperator* getOperator(std::string);

  /// Prints the contents of the hash to standard out.
  void printOperatorHash();
};

}

/**
 * @defgroup singletons Singletons
 */
/* @{ */

/// Singleton type for accessing the operator manager class.
typedef Loki::SingletonHolder<Alida::_ALDOperatorManager,
  Loki::CreateUsingNew,
  Loki::DefaultLifetime,
  Loki::SingleThreaded
> ALDOperatorManager;

/* @} */

#endif /* OPERATOR_MANAGER_H_INCLUDED */
